/**
 * Copyright (C) 2012 The SkyTvOS Project
 *
 * Version     Date           Author
 * ─────────────────────────────────────
 *           2016年2月18日         jisheng
 *
 */

package com.coocaa.smartscreen.api;

import java.lang.ref.WeakReference;

import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;

public class SmartAppApi
{
    private static final String SMART_ACTION = "com.tianci.smartscreen.START";
    private static final String SMART_PKG = "com.tianci.smartscreen";
    private WeakReference<Context> wContext = null;
    private boolean Debug = true;

    public SmartAppApi(Context context)
    {
        wContext = new WeakReference<Context>(context);
    }

    /**
     * @Description 退出的时候。比如activity、service，onDestory()方法里面
     *              必须保证调用到，防止存在内存泄露。
     *              
     * @date 2016年3月1日
     */
    public void cleanUp()
    {
        if (wContext != null && wContext.get() != null)
        {
            Intent intent = new Intent(SMART_ACTION);
            intent.setPackage(SMART_PKG);
            intent.putExtra("cmd", "cleanUp");
            wContext.get().sendBroadcast(intent);
            if (Debug)
            {
                System.out.println("[ cleanUp() ]");
            }
        } else
        {
            System.out.println("[ cleanUp()  wContext == null]");
        }

    }

    /**
     * @Description  
     * @param debug true,表示会输出打野，false，不会输出打印
     * @date 2016年3月2日
     */
    public void setDebug(boolean debug)
    {
        this.Debug = debug;
    }

    /**
     * @Description  和ble设备连接状态发生改变的时候，调用该方法。 
     * @param addr   设备的地址
     * @param state Can be one of
     *                  {@link BluetoothProfile#STATE_DISCONNECTED} or
     *                  {@link BluetoothProfile#STATE_CONNECTED}
     * @date 2016年2月18日
     */
    public void onStatusChange(String addr, int state)
    {
        if (wContext != null && wContext.get() != null)
        {
            Intent intent = new Intent(SMART_ACTION);
            intent.setPackage(SMART_PKG);
            intent.putExtra("cmd", "onStatusChange");
            intent.putExtra("pkg", wContext.get().getPackageName());
            intent.putExtra("addr", addr);
            intent.putExtra("status", state);
            wContext.get().sendBroadcast(intent);
            if (Debug)
            {
                System.out.println("[ onStatusChange(String addr,int state):addr=" + addr
                        + ",state=" + state + "  ]");
            }

        } else
        {
            if (Debug)
            {
                System.out.println("[ onStatusChange(String addr,int state):addr=" + addr
                        + ",state=" + state + " wContext ==null ]");
            }
        }

    }

    /**
     * @Description  ble设备休眠之后，多少分钟之后，执行退出操作。
     * @param minutes ag:mintes == 5,表示设备休眠之后，5分钟后就会退出，回到之前状态。
     * @date 2016年2月18日
     */
    public void setExitTime(int minutes)
    {

        if (wContext != null && wContext.get() != null)
        {
            Intent intent = new Intent(SMART_ACTION);
            intent.setPackage(SMART_PKG);
            intent.putExtra("cmd", "setAfterMinutesExit");
            intent.putExtra("pkg", wContext.get().getPackageName());
            intent.putExtra("minutes", minutes);
            wContext.get().sendBroadcast(intent);
            if (Debug)
            {
                System.out.println("[ setExitTime(int minutes):minutes=" + minutes + " ]");
            }
        } else
        {
            if (Debug)
            {
                System.out.println("[ setExitTime(int minutes):minutes=" + minutes
                        + " wContext=null]");
            }
        }

    }

    /**
     * @Description  回到之前状态
     *               在本次由系统自动唤醒后，用户已经使用完毕，需要回到上一个状态调用。
     *               使用场景：电视处于A状态，用户体重测量完毕，用户也查看完毕，此时理想状态是回到A状态。这种情况下调用该方法
     * @date 2016年2月18日
     */
    public void backToLastStatus()
    {

        if (wContext != null && wContext.get() != null)
        {
            Intent intent = new Intent(SMART_ACTION);
            intent.setPackage(SMART_PKG);
            intent.putExtra("cmd", "backToLastStatus");
            intent.putExtra("pkg", wContext.get().getPackageName());
            wContext.get().sendBroadcast(intent);
            if (Debug)
            {
                System.out.println("[ backToLastStatus() ]");
            }
        } else
        {
            if (Debug)
            {
                System.out.println("[ backToLastStatus() wContext =null]");
            }
        }

    }

}
